#!/usr/local/bin/perl -w

# files to do file name substitution

$script[0] = "ifile.h";
$script[1] = "knowledge_base.perl";
$script[2] = "news2mail.perl";

# gets parent working directory (tries 3 different methods)

if ($_ = `pwd`)
{
    s/\n$//;
    print "PWD (\`pwd\`) = $_\n";
    s/\n$//;    # removes feedline
    $cur_dir = $_;
}
elsif (-x "/bin/pwd") {
    open(PWD, "/bin/pwd | ")
	|| die "Couldn't run /bin/pwd (even though it said I could!): $!\n";
    $_ = <PWD>;
    print "PWD (/bin/pwd) = $_";
    s/\n$//;    # removes feedline
    $cur_dir = $_;
} else {
    print "No 'pwd' executable found\n";
    open(PWD, "echo \$PWD | ")
	|| die "Couldn't get environment variable PWD (echo $PWD): $!\n";
    $_ = <PWD>;
    s/\n$//;    # removes feedline
    print "PWD (echo \$PWD) = $_";
    $cur_dir = $_;
}

close(PWD);

# if $cur_dir doesn't begin with '/', it's probably empty (need to ask user)
if (! $cur_dir =~ m#^/#)
{
    print "What directory did you execute install.perl in?\n";
    $cur_dir = <STDIN>;
    chop $cur_dir;
}

# add an ending '/' if $cur_dir doesn't already have it
if ($cur_dir =~ m/[^\/]$/)
{
    $cur_dir .= "/";
}

# the current installation directory of the binaries is the same as the
#   current directory.  In the future, this might be changed so that
#   the executables are moved to a global bin directory.
$install_dir = $cur_dir;

# if necessary, append the path with a '/'
if ($cur_dir =~ m/[^\/]$/) { $cur_dir .= "/"; }

# get the location of mh binaries

print "\n";

$refile_dir = &get_dir("refile", "bin");
print "refile = $refile_dir\n";
$rcvstore_dir = &get_dir("rcvstore", "lib");
print "rcvstore = $rcvstore_dir\n";

# if this script executed, it must have the right path for perl!
if (open(INSTALL, $0))
{
    $line = <INSTALL>;
    $line =~ m/^#!(\S+)perl\s/;
    $perl_dir = $1;
    print "perl = $perl_dir\n";
} 
else
{
    $perl_dir = &get_dir("perl", "bin");
    print "perl = $perl_dir\n";
}

print "\n";

# gets the user's home directory to perform modification of .maildelivery file

$home_dir = "";

while (! -d $home_dir) 
{
       print "Where is your home directory?  ";
       $home_dir = <STDIN>;
       chop $home_dir;

       if (! -d $home_dir) { print "'$home_dir' isn't a directory!\n"; }
}

if ($home_dir =~ m/[^\/]$/) { $home_dir .= "/"; }
print "Home dir = $home_dir\n\n";

# read in old .maildelivery file

$modify_maildelivery = "true";
if (open(MAIL, "< $home_dir.maildelivery")) 
{
    @mail = <MAIL>;
    close(MAIL);
}
else 
{
    print "No $home_dir.maildelivery file found.\n";
    print "Should I create it for you? (ifile will not work automatically unless this file is created)\n";
    $response = <STDIN>;
    if ($response =~ m/^[^y]/i)
    {
	print "You decided not to create a .maildelivery file.\n";
	print "Mail will NOT automatically be filtered through ifile via slocal.\n";
	$modify_maildelivery = "false";
    }
}

if ($modify_maildelivery ne "false")
{
    print "Changing ifilter entry in .maildelivery file to reflect new ifilter location.\n";
    print "    (does not change the position of the ifilter entry)\n";
    
    # creating new .maildelivery file
    # does not affect ordering of entries in .maildelivery file (previous
    #   versions put ifilter line at the top of the file)

    $wrote_ifilter = "false";
    
    if (open(MAIL, "> $home_dir.maildelivery"))
    {
	foreach $line (@mail)
	{
	    if ($wrote_ifilter eq "false")
	    {
		# replaces "* - ^ ? /old/install/path/ifilter" with
		#  "* - ^ ? /new/install/path/ifilter"
		if ($line =~ s/\* \- \^ \? \/\S+ifilter/\* \- \^ \? ${cur_dir}ifilter/)
		{
		    $wrote_ifilter = "true";
		}
            }
	    print MAIL $line;
	}
	close(MAIL);
	
	# if there was no ifilter line in .maildelivery, we need to make one!
	
	if ($wrote_ifilter eq "false")
	{
	    open(MAIL, "> $home_dir.maildelivery");
	    print MAIL "\* \- \^ \? ${cur_dir}ifilter\n";
	    foreach $line (@mail) { print MAIL $line; }
	}
	
	$write_maildelivery = "true";
	
	# 0644 permissions required by slocal
	chmod(0644, "$home_dir.maildelivery");
    } 
    else
    { 
	warn "** Could not create $home_dir.maildelivery file!\n"; 
	$write_maildelivery = "false";
    }
}

print "Modifying ".join(", ", @script)." to reflect new binary locations...\n";

foreach $file (@script)
{
    open(IN, "< $cur_dir$file") || die "Could not open $cur_dir$file for reading\n";
	
    @stuff = <IN>;
	
    open(OUT, "> $cur_dir$file") || 
	die "Could not open $cur_dir$file for writing\n";
	
    foreach $line (@stuff)
    {
	$line =~ s/#!\/\S*perl/#!${perl_dir}perl/;
	$line =~ s/#define MH_RCVSTORE \"[^\"]*\"/#define MH_RCVSTORE \"${rcvstore_dir}rcvstore\"/;
	$line =~ s/#define MH_REFILE \"[^\"]*\"/#define MH_REFILE \"${refile_dir}refile.bak\"/;
	$line =~ s/\$IFILTER = \"[^\"]*\"/\$IFILTER = \"${install_dir}ifilter\"/;
        print OUT $line;
    }
    close(IN);
    close(OUT);
    chmod 0755, $cur_dir.$file;
}

# This MUST be done before the MH refile link is made, otherwise the refile
# link will consider the irefile executable to not be executable

print "\nCompiling C binaries...\n";

system "make all";

# check to see whether 'refile' is an actual binary or just a link.  If it
#   is a link, we don't want to try to back it up.  If it is an actual binary,
#   move it to refile.bak

print "\nBacking up mh refile binary...\n";

if (-l "${refile_dir}refile") {
    warn "** ${refile_dir}refile is a link, I can't back it up!\n";
    warn "    (if this is an upgrade, you may safely ignore the above warning)\n";
    print "Press [Enter] to remove refile link (or Control-C to abort).\n";
    <STDIN>;
    print "Removing ${refile_dir}refile...\n";
    system "rm", "${refile_dir}refile";
} else {
    system "mv", "${refile_dir}refile", "${refile_dir}refile.bak";
}

# link 'refile' to the location of irefile
print "Creating link from ${refile_dir}refile to irefile...\n";

symlink("${cur_dir}irefile", "${refile_dir}refile");

# self-reminder:
# IMPLEMENT THE FOLLOWING ONCE IFILE CAN BE INSTALLED BY A REGULAR USER!
# if this appears to be the user's first installation, ask if they want to
#   automatically build the knowledge base
# if (! -r $home_dir.".idata")
# {
#     print "Do you wish me to run knowledge_base.perl to automatically make a model of\n";
#     print "    your folder preferences?  (highly recommended - takes a few minutes to run\n";
#     $ans = <STDIN>;
#     if ($ans =~ m/^y/i)
#     {
# 	system "./knowledge_base.perl";
#     }
# }

# if the writing of .maildelivery didn't work, tell the user what to do.

if ($write_maildelivery eq "false") {
    print "\nPlease add the following line to the top of your .maildelivery";
    print "file:\n(it is necessary for anyone who wishes to use this";
    print "filtering system)\n\n";
    print "* - ^ ? ${cur_dir}ifilter\n";
}

print "\nIf you are using EXMH, remember to set your default incorporate method to\n";
print "    presort.\n";
print "If you are using MH, make sure to have your mail filtered by slocal\n";

print "\nAll done!\n";


sub get_dir {

    local($binary, $sub_dir) = @_;

    # searches the likely directories for the binary...

    if (-x "/usr/$sub_dir/mh/$binary" || -l "/usr/$sub_dir/mh/$binary") 
    {
	$binary_dir = "/usr/$sub_dir/mh/";
	print "Found $binary in /usr/$sub_dir/mh/\n";
    } 
    elsif (-x "/usr/$sub_dir/$binary" || -l "/usr/$sub_dir/$binary")
    {
	$binary_dir = "/usr/$sub_dir/";
	print "Found $binary in /usr/$sub_dir/\n";
    }
    elsif (-x "/opt/$sub_dir/mh/$binary" || -l "/opt/$sub_dir/mh/$binary") 
    {
	$binary_dir = "/opt/$sub_dir/mh/";
	print "Found $binary in /opt/$sub_dir/mh/\n";
    }
    elsif (-x "/usr/local/$sub_dir/$binary" || -l "/usr/local/$sub_dir/$binary") 
    {
	$binary_dir = "/usr/local/$sub_dir/";
	print "Found $binary in /usr/local/$sub_dir/\n";
    }
    elsif (-x "/usr/local/$sub_dir/mh/$binary" || -l "/usr/local/$sub_dir/mh/$binary")
    {
	$binary_dir = "/usr/local/$sub_dir/mh/";
	print "Found $binary in /usr/local/$sub_dir/mh/\n";
    }
    else
    {
	print "$binary not found in /usr/$sub_dir/mh/ /usr/local/$sub_dir/\n";
	print "    /usr/local/$sub_dir/mh/ /usr/$sub_dir/ /opt/$sub_dir/mh/\n";

	print "What directory is your $binary binary in?  ";
	
	$binary_dir = <STDIN>;
	chop $binary_dir;

	# add ending '/' if necessary
	if ($binary_dir =~ m/[^\/]$/) { $binary_dir .= "/"; }
	
	if (! ((-l $binary_dir.$binary) || (-x $binary_dir.$binary))) {
	    die "$binary_dir$binary is not an executable binary\n";
	}

	print "$binary dir = $binary_dir\n";
    }

    $binary_dir;
}
